IR.AddListener(IR.EVENT_START, 0, function()
{
   var device = GlobalCache("GC-100-12:All Feedbacks (TCP:4998)");
});


// Переменные, которые использует драйвер
// Devices     - информация о внутренних модулях
// Network     - информация о сетевых настройках
// LastError   - текст последней ошибки
// StateX:Y    - состояние коннектора с адресом X:Y (для Relay и Ir Sensor)
// ModeX:Y     - режим коннектора с адресом X:Y (для Serial и IR)

// Сообщения об ошибках
var GlobalCacheErrors = [
   "(0)",
   "(1) Time out occurred because carriage return <CR> not received. The request was now processed.",
   "(2) Invalid module address(module does not exist) received when attempting to ascertain the version number (getversion).",
   "(3) Invalid module address (module does not exist).",
   "(4) Invalid connector address.",
   "(5) Connector address 1 is set up as “sensor in” when attempting to send an IR command.",
   "(6) Connector address 2 is set up as “sensor in” when attempting to send an IR command.",
   "(7) Connector address 3 is set up as “sensor in” when attempting to send an IR command.",
   "(8) Offset is set to an even transition number, but should be set to an odd transition number in the IR command.",
   "(9) Maximum number of transitions exceeded (256 total on/off transitions allowed).",
   "(10) Number of transitions in the IR command is not even (the same number of on and off transitions is required).",
   "(11) Contact closure command sent to a module that is not a relay.",
   "(12) Missing carriage return. All commands must end with a carriage return.",
   "(13) State was requested of an invalid connector address, or the connector is programmed as IR out and not sensor in.",
   "(14) Command sent to the unit is not supported by the GC-100.",
   "(15) Maximum number of IR transitions exceeded. (SM_IR_INPROCESS).",
   "(16) Invalid number of IR transitions (must be an even number).",
   "(17)",
   "(18)",
   "(19)",
   "(20) test ",
   "(21) Attempted to send an IR command to a non-IR module.",
   "(22)",
   "(23) Command sent is not supported by this type of module."
];

//---------------------------------------------------------
// Объект GlobalCache
//---------------------------------------------------------
function GlobalCache(name)
{
   var timer = 0;
   var modules = [];
   var queue = [];
   var device = IR.GetDevice(name);
   
   //------------------------------------------------------
   // Событие: соединение установлено
   //------------------------------------------------------
   IR.AddListener(IR.EVENT_ONLINE, device, function(text)
   { 
      // Запрашиваем информацию о собственных устройствах
      queue.push("getdevices");
      // Запрашиваем настройки сети
      queue.push("get_NET,0:1");
   }, this);

   //------------------------------------------------------
   // Событие: Получен ответ от сервера
   //------------------------------------------------------
   IR.AddListener(IR.EVENT_RECEIVE_TEXT, device, function(text)
   { 
      IR.Log("Text received=" + text);

      // Разделяем строки
      var strings = text.split('\r');
      var string;
      while(string = strings.shift())
      {
         // Делим строку на части
         var parts = string.split(",");
         
         switch(parts[0])
         {
         // Информация о количестве устройств
         case "device":
            var module = parts[1] - 1;
            var slots = parseInt(parts[2].substring(0, 1));
            var type = parts[2].substring(2);
            
            // Добавляем информацию о модулях
            modules[module] = new Array(slots);
            for(var i = 0; i < slots; ++i)
            {
               modules[module][i] = {"type": type};
               var address = parts[1] + ":" + (i + 1);
               switch(type)
               {
               case "RELAY":
                  // Запрашиваем состояние
                  queue.push("getstate," + address);
                  break;
               case "IR":
                  // Запрашиваем режим
                  queue.push("get_IR," + address);
                  break;
               case "SERIAL":
                  // Запрашиваем настройки
                  queue.push("get_SERIAL," + address);
                  break;
               }
            }                  
            break;
            
         // Окончание списка устройств
         case "endlistdevices":
            device.SetFeedback("Devices", text);
            break;
            
         // Состояние
         case "state":
         case "setstate":
         case "statechange":
            device.SetFeedback("State" + parts[1].replace(':', '-'), parts[2]);
            break;
            
         // Режим ИК
         case "IR":
            device.SetFeedback("Mode" + parts[1].replace(':', '-'), parts[2]);
            // Запрашиваем состояние
            if(parts[2].indexOf("SENSOR") >= 0)
               queue.push("getstate," + parts[1]);
            break;

         // Режим порта
         case "SERIAL":
            device.SetFeedback("Mode" + parts[1].replace(':', '-'), parts[2] + "," + parts[3] + "," + parts[4]);
            break;

         // Сетевые настройки
         case "NET":
            device.SetFeedback("Network", text);
            break;
            
         // Ошибка
         default:
            if(string.indexOf("unknowncommand") >= 0)
               device.SetFeedback("LastError", GlobalCacheErrors[string.substring(15)]);
            break;
         }      
      }
   }, this);

   //---------------------------------------------------------
   // Действия каждый цикл
   //---------------------------------------------------------
   IR.SetInterval(100, function(){
    // Посылаем команду из очереди
    var command = queue.shift();
    if(command) device.Send([command,13]);
   }, this);
}